package org.rcisoft.business.overview.dao;

import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.business.overview.entity.EnergyCountM;
import org.rcisoft.business.overview.entity.EnergyPrice;
import org.rcisoft.business.overview.vo.EnergyCountMVo;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * Created by JiChao on 2018/4/11.
 * 概览
 */
@Repository
public interface EnergyCountMRepository extends BaseMapper<EnergyCountM> {

    /**
     * 用量和费用统计，当日或当月
     * @param energyCountM proId：项目id；day：天；mon：月；year：年
     * @return
     */
    @Select("<script>select sum(WATER) as water,sum(WATER_MONEY) as waterMoney," +
            "sum(ELEC) as elec,sum(ELEC_MONEY) as elecMoney," +
            "sum(GAS) as gas,sum(GAS_MONEY) as gasMoney" +
            " from ENERGY_COUNT_M e " +
            "where e.PRO_ID=#{proId} and `year`=#{year}" +
            "<if test=\"day != null\"> and `day`=${day} group by e.`day`</if>" +
            "<if test=\"mon != null\"> and mon=${mon} group by e.mon</if></script>")
    @ResultMap("BaseResultMap")
    EnergyCountM countAmountAndPrice(EnergyCountM energyCountM);

    /**
     * 运行费用统计
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select e.`HOUR`," +
            "<if test=\"type == 1\">sum(e.WATER_MONEY)</if>" +
            "<if test=\"type == 2\">sum(e.ELEC_MONEY)</if>" +
            "<if test=\"type == 3\">sum(e.GAS_MONEY)</if>" +
            " as MONEY from ENERGY_COUNT_M e" +
            " where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=${mon} and e.`DAY`=${day} group by e.`HOUR`+0 asc</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> countDailyPrice(EnergyCountMVo energyCountMVo);

    /**
     * 耗能拆分统计 水统计
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select if(sum(e.WATER) is null, 0, sum(e.WATER)) ENERGY from energy_count_m e " +
            "where e.`DAY`=${day} and e.MON=${mon} and e.`YEAR`=#{year} and e.PRO_ID=#{proId}</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> countEnergySplitWater(EnergyCountMVo energyCountMVo);

    /**
     * 耗能拆分统计 电、气统计
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select d.DEV_NM `NAME`," +
            "<if test=\"type == 2\">sum(e.ELEC) ENERGY </if>" +
            "<if test=\"type == 3\">sum(e.GAS) ENERGY </if>" +
            "from bus_device d,bus_device_meter dm,energy_count_m e " +
            "where d.DEV_NUM=dm.DEV_NUM and dm.MET_NUM=e.MET_NUM and " +
            "e.`DAY`=${day} and e.MON=${mon} and e.`YEAR`=#{year} and e.PRO_ID=#{proId} " +
            "group by d.DEV_NM</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> countEnergySplitElecGas(EnergyCountMVo energyCountMVo);

    /**
     * 运行费用排名
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select a.* from (" +
            "select e.`DAY`," +
            "<if test=\"type == 1\">sum(e.WATER_MONEY)</if>" +
            "<if test=\"type == 2\">sum(e.ELEC_MONEY)</if>" +
            "<if test=\"type == 3\">sum(e.GAS_MONEY)</if>" +
            " as MONEY from ENERGY_COUNT_M e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=${mon}" +
            "<if test=\"hour != null\"> and e.HOUR &lt;= #{hour}</if>" +
            " group by e.`DAY`+0 asc) a order by a.MONEY desc</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> getPriceRank(EnergyCountMVo energyCountMVo);

    /**
     * 用能比较
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select e.`DAY`," +
            "<if test=\"type == 1\">sum(e.WATER)</if>" +
            "<if test=\"type == 2\">sum(e.ELEC)</if>" +
            "<if test=\"type == 3\">sum(e.GAS)</if>" +
            " as ENERGY from energy_count_m e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=${mon} group by e.`DAY`+0 asc</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> getEnergyCompare(EnergyCountMVo energyCountMVo);

    /**
     * 按月统计水电气能耗
     * @param energyCountM proId：项目主键，year：年份
     * @return
     */
    @Select("<script>select e.MON,sum(e.WATER) as WATER,sum(e.ELEC) as ELEC,sum(e.GAS) as GAS from energy_count_m e" +
            " where e.PRO_ID=#{proId} and e.`YEAR`=#{year} group by e.MON+0 asc</script>")
    @ResultMap("BaseResultMap")
    List<EnergyCountM> getEnergyMonth(EnergyCountM energyCountM);

    /**
     * 得到每天水电气的能耗
     * @param energyCountM
     * @return
     */
    @Select("<script>select SUM(e.WATER) WATER,SUM(e.ELEC) ELEC,SUM(e.GAS) GAS from energy_count_m e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=${mon} and e.`DAY`=${day}</script>")
    @ResultMap("BaseResultMap")
    EnergyCountM countEnergyDay(EnergyCountM energyCountM);

    /**
     * 年用能计划
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select e.MON,sum(if(e.WATER_MONEY is null,0,e.WATER_MONEY))+sum(if(e.ELEC_MONEY is null,0,e.ELEC_MONEY))+sum(if(e.GAS_MONEY is null,0,e.GAS_MONEY)) MONEY from energy_count_m e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} group by e.MON+0 asc</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> statEnergyPlanYear(EnergyCountMVo energyCountMVo);

    /**
     * 月用能计划
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select e.`DAY`,sum(if(e.WATER_MONEY is null,0,e.WATER_MONEY))+sum(if(e.ELEC_MONEY is null,0,e.ELEC_MONEY))+sum(if(e.GAS_MONEY is null,0,e.GAS_MONEY)) MONEY from energy_count_m e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=${mon} group by e.`DAY`+0 asc</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> statEnergyPlanMonth(EnergyCountMVo energyCountMVo);

    /**
     * 日用能计划
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select e.`HOUR`,sum(if(e.WATER_MONEY is null,0,e.WATER_MONEY))+sum(if(e.ELEC_MONEY is null,0,e.ELEC_MONEY))+sum(if(e.GAS_MONEY is null,0,e.GAS_MONEY)) MONEY from energy_count_m e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=${mon} and e.`DAY`=${day} group by e.`HOUR`+0 asc</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> statEnergyPlanDay(EnergyCountMVo energyCountMVo);

    /**
     * 能源消耗
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select e.`HOUR`,sum(if(e.WATER is null,0,e.WATER))+sum(if(e.ELEC is null,0,e.ELEC))+sum(if(e.GAS is null,0,e.GAS)) ENERGY " +
            "from energy_count_m e where e.PRO_ID=#{proId} and date_format(e.CREATE_TIME, '%Y-%c-%e')=#{time} group by e.`HOUR`+0 asc</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> energyConsume(EnergyCountMVo energyCountMVo);

}
